@@ -100,10 +100,6 @@
 #include <vips/thread.h>
 #include <vips/internal.h>
 
-#if ENABLE_DEPRECATED
-#include <vips/vips7compat.h>
-#endif
-
 /* abort() on the first warning or error.
  */
 int vips__fatal = 0;
@@ -135,6 +131,40 @@ int vips__leak = 0;
 GQuark vips__image_pixels_quark = 0;
 #endif /*DEBUG_LEAK*/
 
+/* The maximum coordinate (ie. dimension) value we allow. This can be
+ * overridden with the `--vips-max-coord` CLI arg, or the `VIPS_MAX_COORD` env
+ * var.
+ */
+static char *vips__max_coord_arg = NULL;
+
+/**
+ * vips_max_coord_get:
+ *
+ * Return the maximum coordinate value. This can be the default, a value set
+ * set by the `--vips-max-coord` CLI arg, or a value set in the `VIPS_MAX_COORD`
+ * environment variable.
+ *
+ * These strings can include unit specifiers, eg. "10m" for 10 million pixels.
+ * Values above INT_MAX are not supported.
+ *
+ * Returns: The maximum value a coordinate, or image dimension, can have.
+ */
+int
+vips_max_coord_get(void)
+{
+	// CLI overrides env var
+	const char *as_str = vips__max_coord_arg ?
+		vips__max_coord_arg : g_getenv("VIPS_MAX_COORD");
+
+	if (as_str) {
+		guint64 size = vips__parse_size(as_str);
+
+		return VIPS_CLIP(100, size, INT_MAX);
+	}
+	else
+		return VIPS_DEFAULT_MAX_COORD;
+}
+
 /**
  * vips_get_argv0:
  *
@@ -257,7 +287,7 @@ vips_load_plugins(const char *fmt, ...)
 		return;
 
 	va_start(ap, fmt);
-	(void) vips_vsnprintf(dir_name, VIPS_PATH_MAX - 1, fmt, ap);
+	(void) g_vsnprintf(dir_name, VIPS_PATH_MAX - 1, fmt, ap);
 	va_end(ap);
 
 	g_info("searching \"%s\"", dir_name);
@@ -271,7 +301,7 @@ vips_load_plugins(const char *fmt, ...)
 		char path[VIPS_PATH_MAX];
 		GModule *module;
 
-		vips_snprintf(path, VIPS_PATH_MAX - 1,
+		g_snprintf(path, VIPS_PATH_MAX - 1,
 			"%s" G_DIR_SEPARATOR_S "%s", dir_name, name);
 
 		g_info("loading \"%s\"", path);
@@ -316,6 +346,7 @@ set_stacksize(guint64 size)
 	if (pthread_attr_init(&attr) ||
 		pthread_attr_getstacksize(&attr, &cur_stack_size)) {
 		g_warning("set_stacksize: unable to get stack size");
+		pthread_attr_destroy(&attr);
 		return;
 	}
 
@@ -327,6 +358,8 @@ set_stacksize(guint64 size)
 			g_info("set stack size to %" G_GUINT64_FORMAT "k",
 				size / (guint64) 1024);
 	}
+
+	pthread_attr_destroy(&attr);
 #endif /*HAVE_PTHREAD_DEFAULT_NP*/
 }
 
@@ -838,6 +871,9 @@ static GOptionEntry option_entries[] = {
 	{ "vips-concurrency", 0, 0,
 		G_OPTION_ARG_INT, &vips__concurrency,
 		N_("evaluate with N concurrent threads"), "N" },
+	{ "vips-max-coord", 0, 0,
+		G_OPTION_ARG_STRING, &vips__max_coord_arg,
+		N_("maximum coordinate"), NULL },
 	{ "vips-tile-width", 0, G_OPTION_FLAG_HIDDEN,
 		G_OPTION_ARG_INT, &vips__tile_width,
 		N_("set tile width to N (DEBUG)"), "N" },
@@ -930,20 +966,20 @@ extract_prefix(const char *dir, const char *name)
 		char *cwd;
 
 		cwd = g_get_current_dir();
-		vips_snprintf(edir, VIPS_PATH_MAX,
+		g_snprintf(edir, VIPS_PATH_MAX,
 			"%s" G_DIR_SEPARATOR_S "%s", cwd, dir);
 		g_free(cwd);
 	}
 	else {
-		vips_strncpy(edir, dir, VIPS_PATH_MAX);
+		g_strlcpy(edir, dir, VIPS_PATH_MAX);
 	}
 
 	/* Chop off the trailing prog name, plus the trailing
 	 * G_DIR_SEPARATOR_S.
 	 */
-	if (!vips_ispostfix(edir, name))
+	if (!g_str_has_suffix(edir, name))
 		return NULL;
-	vips_strncpy(vname, edir, VIPS_PATH_MAX);
+	g_strlcpy(vname, edir, VIPS_PATH_MAX);
 	vname[strlen(edir) - strlen(name) - 1] = '\0';
 
 	/* Remove any "/./", any trailing "/.", any trailing "/".
@@ -953,16 +989,16 @@ extract_prefix(const char *dir, const char *name)
 				vname + i))
 			memmove(vname + i, vname + i + 2,
 				strlen(vname + i + 2) + 1);
-	if (vips_ispostfix(vname, G_DIR_SEPARATOR_S "."))
+	if (g_str_has_suffix(vname, G_DIR_SEPARATOR_S "."))
 		vname[strlen(vname) - 2] = '\0';
-	if (vips_ispostfix(vname, G_DIR_SEPARATOR_S))
+	if (g_str_has_suffix(vname, G_DIR_SEPARATOR_S))
 		vname[strlen(vname) - 1] = '\0';
 
 	g_info("canonicalised path = \"%s\"", vname);
 
 	/* Ought to be a "/bin" at the end now.
 	 */
-	if (!vips_ispostfix(vname, G_DIR_SEPARATOR_S "bin"))
+	if (!g_str_has_suffix(vname, G_DIR_SEPARATOR_S "bin"))
 		return NULL;
 	vname[strlen(vname) - strlen(G_DIR_SEPARATOR_S "bin")] = '\0';
 
@@ -985,7 +1021,7 @@ scan_path(char *path, const char *name)
 
 		/* Form complete path.
 		 */
-		vips_snprintf(str, VIPS_PATH_MAX,
+		g_snprintf(str, VIPS_PATH_MAX,
 			"%s" G_DIR_SEPARATOR_S "%s", p, name);
 
 		g_info("looking in \"%s\" for \"%s\"",
@@ -1021,12 +1057,12 @@ find_file(const char *name)
 		/* Windows always searches '.' first, so prepend cwd to path.
 		 */
 		dir = g_get_current_dir();
-		vips_snprintf(full_path, VIPS_PATH_MAX,
+		g_snprintf(full_path, VIPS_PATH_MAX,
 			"%s" G_SEARCHPATH_SEPARATOR_S "%s", dir, path);
 		g_free(dir);
 	}
 #else  /*!G_OS_WIN32*/
-	vips_strncpy(full_path, path, VIPS_PATH_MAX);
+	g_strlcpy(full_path, path, VIPS_PATH_MAX);
 #endif /*G_OS_WIN32*/
 
 	if ((prefix = scan_path(full_path, name)))
@@ -1082,7 +1118,7 @@ guess_prefix(const char *argv0, const char *name)
 		char *resolved;
 
 		dir = g_get_current_dir();
-		vips_snprintf(full_path, VIPS_PATH_MAX,
+		g_snprintf(full_path, VIPS_PATH_MAX,
 			"%s" G_DIR_SEPARATOR_S "%s", dir, argv0);
 		g_free(dir);
 
